#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "lac_param.h"
#include "gen_test_data.h"
#include "pke_api.h"
#include "kem_api.h"
#include "kex_api.h"
#include "ake_api.h"
#include "rand.h"
//generate test data  of pke
int testdata_pke()
{
	unsigned char pk[CRYPTO_PUBLICKEYBYTES];
	unsigned char sk[CRYPTO_SECRETKEYBYTES];
	unsigned char m[CRYPTO_BYTES],c[CRYPTO_CIPHERTEXTBYTES];
	unsigned long long clen,mlen;
	int i;
	
	mlen=CRYPTO_BYTES;
	pke_keygen(pk,sk);
	random_bytes(m,CRYPTO_BYTES);
	pke_enc(pk,m,mlen,c,&clen);
	
	FILE *fp;
	//create kat file
	printf("\ntest data of pke is generated in test_data_pke.txt\n");
	fp=fopen("test_data_pke.txt","w");
	if(fp==NULL)
	{
		return -1;
	}
	//wirte arrays
	fprintf(fp,"PK\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk[i]>>4);
		fprintf(fp,"%x",pk[i]&0xf);
	}
	fprintf(fp,"\n\nSK\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk[i]>>4);
		fprintf(fp,"%x",sk[i]&0xf);
	}
	fprintf(fp,"\n\nPlaintext\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",m[i]>>4);
		fprintf(fp,"%x",m[i]&0xf);
	}
	fprintf(fp,"\n\nCiphertext\n");
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c[i]>>4);
		fprintf(fp,"%x",c[i]&0xf);
	}
	
	fclose(fp);
	return 0;
}

//generate test data of kem
int testdata_kem()
{
	unsigned char pk[CRYPTO_PUBLICKEYBYTES];
	unsigned char sk[CRYPTO_SECRETKEYBYTES];
	unsigned char k[CRYPTO_BYTES],c[CRYPTO_CIPHERTEXTBYTES];
	
	int i;
	
	kem_keygen(pk,sk);
	kem_enc(pk,k,c);
	
	FILE *fp;
	//create kat file
	printf("\ntest data of kem is generated in test_data_kem.txt\n");
	fp=fopen("test_data_kem.txt","w");
	if(fp==NULL)
	{
		return -1;
	}
	//wirte arrays
	fprintf(fp,"PK\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk[i]>>4);
		fprintf(fp,"%x",pk[i]&0xf);
	}
	fprintf(fp,"\n\nSK\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk[i]>>4);
		fprintf(fp,"%x",sk[i]&0xf);
	}
	fprintf(fp,"\n\nSession Key\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",k[i]>>4);
		fprintf(fp,"%x",k[i]&0xf);
	}
	fprintf(fp,"\n\nCiphertext\n");
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c[i]>>4);
		fprintf(fp,"%x",c[i]&0xf);
	}
	
	fclose(fp);
	return 0;
}

//generate test data of ke
int testdata_ke()
{
	unsigned char pk[CRYPTO_PUBLICKEYBYTES];
	unsigned char sk[CRYPTO_SECRETKEYBYTES];
	unsigned char k1[CRYPTO_BYTES],k2[CRYPTO_BYTES],c[CRYPTO_CIPHERTEXTBYTES];
	
	int i;
	
	kex_alice_send(pk,sk);
	kex_bob_receive(pk,c,k1);
	kex_alice_receive(pk,sk,c,k2);
	
	
	FILE *fp;
	//create kat file
	fp=fopen("test_data_kex.txt","w");
	printf("\ntest data of kex is generated in test_data_kex.txt\n");
	if(fp==NULL)
	{
		return -1;
	}
	//wirte arrays
	fprintf(fp,"1:Alice Generate PK and SK, send PK to Bob.\nPK\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk[i]>>4);
		fprintf(fp,"%x",pk[i]&0xf);
	}
	fprintf(fp,"\n\nSK\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk[i]>>4);
		fprintf(fp,"%x",sk[i]&0xf);
	}
	fprintf(fp,"\n\n2:Bob receive PK from Alilce, generate Sesssion Key and Ciphertext.\nCiphertext\n");
	
	
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c[i]>>4);
		fprintf(fp,"%x",c[i]&0xf);
	}
	fprintf(fp,"\n\nSession Key output by Bob\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",k1[i]>>4);
		fprintf(fp,"%x",k1[i]&0xf);
	}
	
	fprintf(fp,"\n\n3:Alice receive Ciphertext from Bob, output the session key.\nSession Key output by Alice\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",k2[i]>>4);
		fprintf(fp,"%x",k2[i]&0xf);
	}
	
	fclose(fp);
	return 0;
}

//generate test  data  of ake
int testdata_ake()
{
	unsigned char pk_a[CRYPTO_PUBLICKEYBYTES],pk_b[CRYPTO_PUBLICKEYBYTES],pk[CRYPTO_PUBLICKEYBYTES];
	unsigned char sk[CRYPTO_SECRETKEYBYTES],sk_a[CRYPTO_SECRETKEYBYTES],sk_b[CRYPTO_SECRETKEYBYTES];
	unsigned char k_a[CRYPTO_BYTES],k_b[CRYPTO_BYTES],c_a[CRYPTO_CIPHERTEXTBYTES],c_b[2*CRYPTO_CIPHERTEXTBYTES],k1[CRYPTO_BYTES];
	
	int i;
	
	kem_keygen(pk_a,sk_a);
	kem_keygen(pk_b,sk_b);
	
	ake_alice_send(pk,sk,pk_b,sk_a,c_a,k1);
	ake_bob_receive(pk_b,sk_b,pk_a,pk,c_a,c_b,k_b);
	ake_alice_receive(pk_a,sk_a,pk_b,pk,sk,c_b,k1,k_a);
	
	FILE *fp;
	//create kat file
	fp=fopen("test_data_ake.txt","w");
	printf("\ntest data of ake is generated in test_data_ake.txt\n");
	if(fp==NULL)
	{
		return -1;
	}
	//wirte arrays
	fprintf(fp,"0:Aalice and Bob generate its own public key and private key.\nPK of Alice\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk_a[i]>>4);
		fprintf(fp,"%x",pk_a[i]&0xf);
	}
	fprintf(fp,"\n\nSK of Alice\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk_a[i]>>4);
		fprintf(fp,"%x",sk_a[i]&0xf);
	}
	
	fprintf(fp,"\n\nPK of Bob\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk_b[i]>>4);
		fprintf(fp,"%x",pk_b[i]&0xf);
	}
	fprintf(fp,"\n\nSK of Bob\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk_b[i]>>4);
		fprintf(fp,"%x",sk_b[i]&0xf);
	}
	
	fprintf(fp,"\n\n1:Alice Generate temporary PK and SK, send PK and a kem ciphertext encrypted by Bob's PK to Bob.\nTemporary PK\n");
	for(i=0;i<CRYPTO_PUBLICKEYBYTES;i++)
	{
		fprintf(fp,"%x",pk[i]>>4);
		fprintf(fp,"%x",pk[i]&0xf);
	}
	fprintf(fp,"\n\nTempoprary SK\n");
	for(i=0;i<CRYPTO_SECRETKEYBYTES;i++)
	{
		fprintf(fp,"%x",sk[i]>>4);
		fprintf(fp,"%x",sk[i]&0xf);
	}
	fprintf(fp,"\n\nCiphertext generated by Alice\n");
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c_a[i]>>4);
		fprintf(fp,"%x",c_a[i]&0xf);
	}
	fprintf(fp,"\n\n2:Bob receive temporary PK from Alilce, generate ciphertext by using Alice's PK and temporary PK. \nCiphertext generate with Alices'PK\n");
	
	
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c_b[i]>>4);
		fprintf(fp,"%x",c_b[i]&0xf);
	}
	
	fprintf(fp,"\n\nCiphertext generate with temporary PK\n");
	
	for(i=0;i<CRYPTO_CIPHERTEXTBYTES;i++)
	{
		fprintf(fp,"%x",c_b[i+CRYPTO_CIPHERTEXTBYTES]>>4);
		fprintf(fp,"%x",c_b[i+CRYPTO_CIPHERTEXTBYTES]&0xf);
	}
	fprintf(fp,"\n\nSession Key output by Bob\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",k_b[i]>>4);
		fprintf(fp,"%x",k_b[i]&0xf);
	}
	
	fprintf(fp,"\n\n3:Alice receive Ciphertext from Bob, output the session key.\nSession Key output by Alice\n");
	for(i=0;i<CRYPTO_BYTES;i++)
	{
		fprintf(fp,"%x",k_a[i]>>4);
		fprintf(fp,"%x",k_a[i]&0xf);
	}
	
	fclose(fp);
	return 0;
}

